require("util")
require("tilevid")
require("axe")
require("turkey")

function scene_view_setup(scene,x,y)
    scene.view.x = x - scene.view.w/2
    scene.view.y = y - scene.view.h/2

    local tilemap = scene.tilemap
    local data = scene.data
    local w = #data[1]
    local h = #data
    
    local xmin = w
    local xmax = 0
    local ymin = h
    local ymax = 0
    
    local tx = math.floor(x/GAME.TW)
    local ty = math.floor(y/GAME.TH)
    for x=tx,tx+w do
        local t = GAME.tiles[tilemap:get(x,ty)]
        if t.edge then break end
        xmax = max(x,xmax)
    end
    y = ty
    for x=tx,tx-w,-1 do
        local t = GAME.tiles[tilemap:get(x,ty)]
        if t.edge then break end
        xmin = min(x,xmin)
    end
    for y=ty,ty+h do
        local t = GAME.tiles[tilemap:get(tx,y)]
        if t.edge then break end
        ymax = max(y,ymax)
    end
    for y=ty,ty-h,-1 do
        local t = GAME.tiles[tilemap:get(tx,y)]
        if t.edge then break end
        ymin = min(y,ymin)
    end
    
--     print("bounds:"..xmin.." "..ymin.." "..xmax.." "..ymax)
    scene.bounds.x = xmin * GAME.TW
    scene.bounds.y = ymin * GAME.TH
    scene.bounds.w = (xmax-xmin+1) * GAME.TW
    scene.bounds.h = (ymax-ymin+1) * GAME.TH
end

function Animate_new(scene,tile)
    scene.tilemap:set(tile.x,tile.y,0)
    local obj = Animate(scene)
    scene:add(obj)
end

class("Animate",Sprite)
function Animate:init(scene)
    Sprite.init(self,"blank",Rect(0,0,0,0))
    self.t = 0
end
function Animate:loop(scene,t)
    self.t = self.t + t
    for n,tile in pairs(GAME.tiles) do
        if tile.frames ~= nil then
            local n = math.floor(self.t / (#tile.frames * tile.duration)) % (#tile.frames)
            tile.image = tile.frames[n+1]
        end
    end
end


function _game_tiles_near(rect)
    local left = round(rect:left())
    local right = round(rect:right())
    local top = round(rect:top())
    local bottom = round(rect:bottom())
    local xs = {} 
    for x=left,right,GAME.TW do xs[#xs+1] = x end
    if xs[#xs] ~= right then xs[#xs+1] = right end
    local ys = {}
    for y=top,bottom,GAME.TH do ys[#ys+1] = y end
    if ys[#ys] ~= bottom then ys[#ys+1] = bottom end
    local r = {}
    for n,y in pairs(ys) do for n,x in pairs(xs) do
        local tx = math.floor(x/GAME.TW)
        local ty = math.floor(y/GAME.TH)
        local tr = Rect(tx*GAME.TW,ty*GAME.TH,GAME.TW,GAME.TH)
        tr.tx = tx
        tr.ty = ty
        r[#r+1] = tr
    end end
    return r
end
    
function game_tiles_hit(scene,obj)
    local _rect = obj._rect
    local rect = obj.rect:copy()
    obj.rect.x = _rect.x
    obj.rect.y = _rect.y
    
    -- x axis
    obj.rect.x = rect.x
    game_tiles_hit_x(scene,obj._rect,obj.rect)
    
    -- y axis
    obj.rect.y = rect.y
    game_tiles_hit_y(scene,obj._rect,obj.rect)
end

function game_tiles_hit_x(scene,_rect,rect)
    local dx = rect.x - _rect.x
    for n,tr in pairs(_game_tiles_near(rect)) do
        local t = GAME.tiles[scene.tilemap:get(tr.tx,tr.ty)]
        if rect:colliderect(tr) then
            if t.left and tr:left() < rect:right() and dx > 0 then rect:right(tr:left()) ; return end
            if t.right and tr:right() > rect:left() and dx < 0 then rect:left(tr:right()) ; return end
        end
    end
end

function game_tiles_hit_y(scene,_rect,rect)
    local dy = rect.y - _rect.y
    for n,tr in pairs(_game_tiles_near(rect)) do
        local t = GAME.tiles[scene.tilemap:get(tr.tx,tr.ty)]
        if rect:colliderect(tr) then
            if t.top and tr:top() < rect:bottom() and dy > 0 then rect:bottom(tr:top()) ; return end
            if t.bottom and tr:bottom() > rect:top() and dy < 0 then rect:top(tr:bottom()) ; return end
        end
    end
end

-- function _game_tiles_hit_x(scene,obj)
--     local rect = obj.rect
--     for n,tr in pairs(_game_tiles_near(rect)) do
--         local t = GAME.tiles[scene.tilemap:get(tr.tx,tr.ty)]
--         if rect:colliderect(tr) then
--             if t.left and tr:left() < rect:right() and tr:left() >= rect:left() then rect:right(tr:left()) ; return end
--             if t.right and tr:right() > rect:left() and tr:right() <= rect:right() then rect:left(tr:right()) ; return end
--         end
--     end
-- end
-- 
-- function _game_tiles_hit_y(scene,obj)
--     local rect = obj.rect
--     for n,tr in pairs(_game_tiles_near(rect)) do
--         local t = GAME.tiles[scene.tilemap:get(tr.tx,tr.ty)]
--         if rect:colliderect(tr) then
--             if t.top and tr:top() < rect:bottom() and tr:top() >= rect:top() then rect:bottom(tr:top()) ; return end
--             if t.bottom and tr:bottom() > rect:top() and tr:bottom() <= rect:bottom() then rect:top(tr:bottom()) ; return end
--         end
--     end
-- end
