require("tilevid")
require("gui")
require("json")

function mk_data(w,h)
    local data = {}
    for y=1,h do
        local row = {}
        data[y] = row
        for x=1,w do
            row[x] = 0
        end
    end
    for x=1,w do
        data[1][x] = 1
        data[h][x] = 1
    end
    for y=1,h do
        data[y][1] = 1
        data[y][w] = 1
    end
    return data
end

function do_zoom(scene,cx,cy,zoom)
    cx = cx or scene.view:centerx()
    cy = cy or scene.view:centery()
    local vx = scene.view.x
    local vy = scene.view.y
    local vw = scene.view.w
    local vh = scene.view.h
    scene.view.w = GAME.SW * zoom
    scene.view.h = GAME.SH * zoom
    scene.view.x = cx - scene.view.w * (cx-vx) / vw
    scene.view.y = cy - scene.view.h * (cy-vy) / vh
end

function scene_load(data)
    local w = #data[1]
    local h = #data
    local scene = Scene(Rect(GAME.TW,GAME.TH,w*GAME.TW,h*GAME.TH),Rect(GAME.TW,GAME.TH,GAME.SW,GAME.SH))
    local tilemap = Tilemap(GAME.tiles,data,Rect(GAME.TW,GAME.TH))
    local bkgr = Background("bkgr1",Rect(480,320))
    scene:add(bkgr,"bkgr")
    scene:add(tilemap,"tilemap")
    scene.tilemap = tilemap
    scene.data = data

    scene.view:centerx(scene.bounds:centerx())
    scene.view:centery(scene.bounds:centery())
    
    return scene
end

function scene_init(scene)
    local data = scene.data
    local w = #data[1]
    local h = #data
    for y=1,h do for x=1,w do
        local v = data[y][x] 
        local t = GAME.tiles[v]
        if t ~= nil and t.code_init ~= nil then
            t.x = x
            t.y = y
            t.code_init(scene,t)
        end
    end end
end


function edit_init()
    GAME.edit = GAME.edit or {}

    ---------------------------------------------------------------------
    GAME.modules.edit_menu = GAME.modules.edit_menu or {}
    GAME.modules.edit = GAME.modules.edit_menu
    local obj = GAME.modules.edit_menu
    function obj:init()
        self.gui = Gui(Rect(GAME.SW,GAME.SH))
        local font = Font("font",12)
        for n=1,24 do
            local w = GAME.SW/4
            local h = GAME.SH/6
            local x = ((n-1)%4) * w
            local y = (math.floor((n-1)/4)) * h
            local pad = 4
            self.gui:add(Button(Label(font,tostring(n)),function () event({type='edit',value=n}) end,GBackground("white",0x555555,0xaaaaaa)),Rect(x+pad,y+pad,w-pad*2,h-pad*2))
        end
    end
    function obj:event(e)
        self.gui:event(e)
        if e.type == 'edit' then
            GAME.edit.tile = 2
            GAME.edit.fname = tostring(e.value)..".map"
            GAME.edit.data = json.decode(cuz.file_load(GAME.edit.fname))
            if GAME.edit.data == nil then
                GAME.edit.data = mk_data(GAME.SW*8/GAME.TW,GAME.SH*8/GAME.TH)
            end
            GAME.edit.undo = json.encode(GAME.edit.data)
            
            GAME.edit.scene = scene_load(GAME.edit.data)
            GAME.edit.tilemap = GAME.edit.scene:find("tilemap")
            GAME.edit.zoom = 1.0
            
            GAME.engine:next(GAME.modules.edit_map) 
        end
    end
    function obj:loop(t)
    end
    function obj:paint()
        self.gui:draw()
    end
    
    ---------------------------------------------------------------------
    GAME.modules.edit_map = GAME.modules.edit_map or {}
    local obj = GAME.modules.edit_map
    
    function obj:init(e)
        self.scene = GAME.edit.scene
        self.tilemap = GAME.edit.tilemap
    end
    function obj:event(e)
        if e.type == 'motion' and e.button == 2 then
            self.scene.view.x = self.scene.view.x - e.dx
            self.scene.view.y = self.scene.view.y - e.dy
        end
        
        if e.type == 'down' and e.button == 1 then
            GAME.edit.undo = json.encode(GAME.edit.data)
        end
        
        if e.type == 'keydown' and e.name == 'Z' then
            local tmp = json.encode(GAME.edit.data)
            local data = json.decode(GAME.edit.undo)
            local w = #data[1]
            local h = #data
            for y=1,h do for x=1,w do GAME.edit.data[y][x] = data[y][x] end end
            GAME.edit.undo = tmp
        end
        
        if (e.type == 'down' or e.type == 'motion') and e.button == 1 then
            local d = math.sqrt(e.dx*e.dx+e.dy*e.dy+1)
            for n=1,d do
                local x = e.x + e.dx * n / d
                local y = e.y + e.dy * n / d
                self.tilemap:set(math.floor(x/GAME.TW),math.floor(y/GAME.TH),GAME.edit.tile)
            end
        end
        
        if e.type == 'down' and e.button == 3 then
            GAME.edit.tile = self.tilemap:get(math.floor(e.x/GAME.TW),math.floor(e.y/GAME.TH))
        end
        
        if e.type == 'keydown' and e.name == 'Tab' then
            GAME.engine:next(GAME.modules.edit_pick) 
        end
        
        if e.type == 'keydown' and e.name == 'Escape' then
            cuz.file_save(GAME.edit.fname,json.encode(GAME.edit.data))
            GAME.engine:next(GAME.modules.edit_menu) 
        end
        
        if e.type == 'down' or e.type == 'motion' or e.type == 'up' then
            self.x = e.x
            self.y = e.y
        end
        
        if e.type == 'keydown' and e.text == '1' then
            GAME.edit.zoom = 1.0
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        if e.type == 'keydown' and e.text == '2' then
            GAME.edit.zoom = 1.587
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        if e.type == 'keydown' and e.text == '3' then
            GAME.edit.zoom = 2.520
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        if e.type == 'keydown' and e.text == '4' then
            GAME.edit.zoom = 4.000
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        if e.type == 'keydown' and e.text == '-' then
            GAME.edit.zoom = max(1,min(16,GAME.edit.zoom * 1.587))
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        if e.type == 'keydown' and (e.text == '=' or e.text == '+') then
            GAME.edit.zoom = max(1,min(16,GAME.edit.zoom / 1.587))
            do_zoom(self.scene,self.x,self.y,GAME.edit.zoom)
        end
        
        if e.type == 'keydown' and e.name == 'Space' then
            cuz.file_save(GAME.edit.fname,json.encode(GAME.edit.data))

            GAME.level.data = deepcopy(GAME.edit.data)
            GAME.level.scene = scene_load(GAME.level.data)
            GAME.level.next = GAME.modules.edit_map
            scene_init(GAME.level.scene)
            GAME.engine:next(GAME.modules.level_play)
        end

    end
    function obj:loop(t)
    end
    function obj:paint()
        self.scene:draw()
    end
    ---------------------------------------------------------------------
    
    GAME.modules.edit_pick = GAME.modules.edit_pick or {}
    local obj = GAME.modules.edit_pick
    function obj:init()
        self.gui = Gui(Rect(GAME.SW,GAME.SH))
        local cols = 9
        local rows = 6
        local n = 0
        for _n,tile in pairs(GAME.tiles) do
            if not tile.skip then 
                local w = GAME.SW/cols
                local h = GAME.SH/rows
                local x = ((n)%cols) * w
                local y = (math.floor((n)/cols)) * h
                local pad = 2
                self.gui:add(Button(Image(tile.image,Rect(w-pad*4,h-pad*4)),function () event({type='pick',value=_n}) end,GBackground("white",0x555555,0xaaaaaa)),Rect(x+pad,y+pad,w-pad*2,h-pad*2))
                n = n + 1
            end
        end
    end
    function obj:event(e)
        self.gui:event(e)
        if e.type == 'pick' then
            GAME.edit.tile = e.value
            GAME.engine:next(GAME.modules.edit_map) 
        end
        if e.type == 'keydown' then
            GAME.engine:next(GAME.modules.edit_map) 
        end
    end
    function obj:loop(t)
    end
    function obj:paint()
        self.gui:draw()
    end
    
end