require("util")

class("Gui")

function Gui:init(view)
    self.items = {}
    self.view = view
    self.focus = nil
    self._focus = nil
end

function Gui:draw()
    cuz.view_set(self.view.x,self.view.y,self.view.w,self.view.h)
    for n,obj in pairs(self.items) do
        obj:draw(self)
    end
end

function Gui:loop(t)
    self:_focus_set(self.focus)
    for n,obj in pairs(self.items) do
        if obj.loop then obj:loop(t) end
    end
end


function Gui:add(obj,rect)
    obj.rect = rect
    self.items[#self.items+1] = obj
end

function Gui:_focus_set(obj)
    if self._focus then self._focus.focus = false end
    self.focus = obj
    self._focus = obj
    if self.focus then self.focus.focus = true end
end

function Gui:event(e)
    for n,obj in pairs(self.items) do
        if e.type == 'up' or e.type == 'down' or e.type == 'motion' then
            if obj.rect:collidepoint(e) then
                self:_focus_set(obj)
            end
        end
        if obj.focus then
            obj:event(e)
        end
    end
end

class("Font")

function Font:init(name,points)
    self.name = name
    self.points = points
end

function Font:draw(...)
    local arg = {...}
    arg[3] = arg[3] - 3
    cuz.font_draw(self.name,self.points,table.unpack(arg))
end

function Font:size(...)
    return cuz.font_size(self.name,self.points,...)
end

class("Label")

function Label:init(font,text)
    self.font = font
    self.text = text
end

function Label:draw()
    local w,h = self.font:size(self.text,self.rect.w)
    local x = self.rect.x + (self.rect.w-w)/2
    local y = self.rect.y + (self.rect.h-h)/2
    self.font:draw(self.text,x,y,self.rect.w,0)
end

class("Image")

function Image:init(image,size)
    self.image = image
    self.size = size
end

function Image:draw()
    local x = self.rect.x + (self.rect.w-self.size.w)/2
    local y = self.rect.y + (self.rect.h-self.size.h)/2
    cuz.image_draw(self.image,x,y,self.size.w,self.size.h)
end

class("GBackground")

function GBackground:init(image,color,focus_color)
    self.image = image
    self.color = color
    self.focus_color = focus_color
end
function GBackground:draw()
    local color = self.color
    if self.focus then color = self.focus_color end
    if color == nil then color = 0xffffff end
    cuz.image_draw(self.image,self.rect.x,self.rect.y,self.rect.w,self.rect.h,0,0,0,color,1.0,0,0)
end

class("Button")

function Button:init(widget,click,background)
    self.widget = widget
    self.click = click
    self.background = background
end

function Button:draw()
    if self.background then
        self.background.focus = self.focus
        self.background.rect = self.rect
        self.background:draw()
    end
    self.widget.focus = self.focus
    self.widget.rect = self.rect
    self.widget:draw()
end

function Button:event(e)
    if e.type == 'up' then self.click() end
    if e.type == 'keydown' and (e.name == 'Space' or e.name == 'Return') then self.click() end
end
