
function Turkey_new(scene,tile)
    local x = tile.x
    local y = tile.y
    scene.tilemap:set(x,y,0)
    x = x * GAME.TW
    y = y * GAME.TH
    local obj = Turkey(scene,x,y)
    scene:add(obj,"player")
    scene.player = obj
    
    scene_view_setup(scene,x,y)
end

class("Turkey",Sprite)

function Turkey:init(scene,x,y)
    Sprite.init(self,"turkey-0",Rect(x+3,y+4,10,12),Rect(-3,-4,16,16))
    self.dx = 0
    self.dy = 0
    self.vy = 0
    self.fx = 1
    self.turbo = false
    self.t = 0
    self.live = true
    self._rect = self.rect:copy()
    self.stand = false
    self.respawn = self.rect:copy()
    self.invincible = 0
    
    self.g_time = 0
    self.g_coins = 0
    self.g_bonus = 0
    self.g_deaths = 0
    self.g_total = 0

    local data = scene.data
    local w = #data[1]
    local h = #data
    for y=1,h do for x=1,w do
        local v = data[y][x] 
        local t = GAME.tiles[v]
        if t.coin then self.g_total = self.g_total + 1 end
    end end

    cuz.sample_play("sfx-respawn")
end

function Turkey:event(scene,e)
    if not self.live then return end
    
    if e.type == 'keydown' and e.name == 'Up' then self.dy = -1 end
    if e.type == 'keyup' and e.name == 'Up' then self.dy = max(0,self.dy) end
    if e.type == 'keydown' and e.name == 'Down' then self.dy = 1 end
    if e.type == 'keyup' and e.name == 'Down' then self.dy = min(0,self.dy) end
    if e.type == 'keydown' and e.name == 'Right' then self.dx = 1 end
    if e.type == 'keyup' and e.name == 'Right' then self.dx = min(0,self.dx) end
    if e.type == 'keydown' and e.name == 'Left' then self.dx = -1 end
    if e.type == 'keyup' and e.name == 'Left' then self.dx = max(0,self.dx) end
    
    if (e.type == 'keydown' or e.type == 'keyup') and e.name == 'Up' then e.name = 'Z' end
    
    if e.type == 'keydown' and e.name == 'Z' then
        if self.stand then
            cuz.sample_play("sfx-jump")
            self.vy = -185
            self.stand = false
        end
    end
    
    if e.type == 'keydown' and e.name == 'X' then
        cuz.sample_play("sfx-turbo")
        self.turbo = true
    end
--     if e.type == 'keyup' and e.name == 'X' then self.turbo = false end
end

function Turkey:kill()
    if not self.live then return end
    if self.invincible > 0 then return end
    cuz.sample_play("sfx-death")
    self.g_deaths = self.g_deaths + 1
    self.image = 'splat-0'
    self.t = 0
    self.live = false
end

function Turkey:loop(scene,t)
    self.g_time = self.g_time + t
    
    if not self.live then
        self.dx = 0
        self.turbo = false
        if self.t > 1.5 then -- respawn
            cuz.sample_play("sfx-respawn")
            self.rect = self.respawn:copy()
            self._rect = self.rect:copy()
            self.live = true
            self.t = 0
            self.invincible = 0.5
            self.rect.w = abs(self.rect.w)
        end
    end
    
    if self.invincible > 0 then self.invincible = self.invincible - t end
    
    local mult = 1
    local dx = self.dx
    if self.turbo then
        mult = 6
        if dx == 0 then dx = self.fx end
    end
    local vx = 80 * mult
    local va = 8
 
    
    if self.live and dx ~= 0 then
        self.t = self.t + t * mult
    end
    
    if self.dx ~= 0 then
        self.shape.w = sign(self.dx) * abs(self.shape.w)
        self.fx = self.dx
    end
    
    self.image = 'turkey-'..(math.floor(self.t * 5) %2)
    
    if not self.live then
        local d = 0.25
        self.image = 'splat-'..math.min(3,math.floor((self.t / d)*4))
        self.t = self.t + t
        if self.t > d then self.image = 'blank' end
    end
    
    self.rect.x = self.rect.x + dx * t * vx
    self.vy = self.vy + va
    if self.turbo then self.vy = min(self.vy,0) end
    self.rect.y = self.rect.y + self.vy * t
    
    local rect = self.rect:copy()
    game_tiles_hit(scene,self)
    if rect.y ~= self.rect.y then
        self.vy = 0
    end
    if rect.x ~= self.rect.x then
        self.turbo = false
    end
    
    local r1 = self.rect:copy()
    local r2 = self.rect:copy() ; r2.y = r2.y + 1
    local _r2 = r2:copy()
    game_tiles_hit_y(scene,r1,r2)
    self.stand = (r2.y ~= _r2.y)
        
    if self.live then
        local tx = math.floor(self.rect:centerx()/GAME.TW)
        local ty = math.floor(self.rect:centery()/GAME.TH)
        local t = GAME.tiles[scene.tilemap:get(tx,ty)]
        
        if t.coin then 
            self.g_coins = self.g_coins + 1
            cuz.sample_play("sfx-coin")
            scene.tilemap:set(tx,ty,0)
        end
        if t.bonus then 
            self.g_bonus = self.g_bonus + 1
            cuz.sample_play("sfx-bonus")
            scene.tilemap:set(tx,ty,0)
        end
        if t.goal then 
            cuz.sample_play("sfx-goal")
            scene.tilemap:set(tx,ty,0)
            GAME.engine:next(GAME.modules.level_win) 
        end
        if t.kill then
            self.vy = -90
            self:kill()
        end
        
        local tx = math.floor(self.rect:centerx()/GAME.TW)
        local ty = math.floor((self.rect:bottom()+1)/GAME.TH)
        local t = GAME.tiles[scene.tilemap:get(tx,ty)]
        if self.stand and t.top then self.respawn = self.rect:copy() end
        
        local f = 0.55
        local rect = Rect(round(self.rect.x-GAME.SW*f), round(self.rect.y-GAME.SH*f), GAME.SW*f*2,GAME.SH*f*2)
        scene.view:clamp(rect)
    end
    

    self._rect = self.rect:copy()
end
